@@ -16,14 +16,17 @@
 */
 
 #include "backend.h"
-#include "debug.h"
 #include "audiooutput.h"
 #include "audiodataoutput.h"
 #ifdef PHONON_EXPERIMENTAL
 #include "videodataoutput.h"
 #endif
 #include "audioeffect.h"
+#include "debug.h"
 #include "mediaobject.h"
+#ifndef PHONON_NO_GRAPHICSVIEW
+#include "videographicsobject.h"
+#endif
 #include "videowidget.h"
 #include "devicemanager.h"
 #include "effectmanager.h"
@@ -39,9 +42,9 @@
 
 #include <cstring>
 
-QT_BEGIN_NAMESPACE
-
+#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
 Q_EXPORT_PLUGIN2(phonon_gstreamer, Phonon::Gstreamer::Backend)
+#endif
 
 namespace Phonon
 {
@@ -54,7 +57,6 @@ Backend::Backend(QObject *parent, const QVariantList &)
         : QObject(parent)
         , m_deviceManager(0)
         , m_effectManager(0)
-        , m_debugLevel(Warning)
         , m_isValid(false)
 {
     // Initialise PulseAudio support
@@ -95,18 +97,16 @@ Backend::Backend(QObject *parent, const QVariantList &)
     setProperty("backendWebsite", QLatin1String("http://phonon.kde.org/"));
 #endif //QT_NO_PROPERTIES
 
-    //check if we should enable debug output
-    QString debugLevelString = qgetenv("PHONON_BACKEND_DEBUG");
-    int debugLevel = debugLevelString.toInt();
-    if (debugLevel > 3) //3 is maximum
+    // Check if we should enable debug output
+    int debugLevel = qgetenv("PHONON_BACKEND_DEBUG").toInt();
+    if (debugLevel > 3) // 3 is maximum
         debugLevel = 3;
-    m_debugLevel = (DebugLevel)debugLevel;
     Debug::setMinimumDebugLevel((Debug::DebugLevel)((int) Debug::DEBUG_NONE - 1 - debugLevel));
 
     if (wasInit) {
         m_isValid = checkDependencies();
         gchar *versionString = gst_version_string();
-        logMessage(QString("Using %0").arg(versionString));
+        debug() << "Using" << versionString;
         g_free(versionString);
     }
 
@@ -122,6 +122,8 @@ Backend::~Backend()
 {
     if (GlobalSubtitles::self)
         delete GlobalSubtitles::self;
+    if (GlobalAudioChannels::self)
+        delete GlobalAudioChannels::self;
     delete m_effectManager;
     delete m_deviceManager;
     PulseSupport::shutdown();
@@ -164,15 +166,19 @@ QObject *Backend::createObject(BackendInterface::Class c, QObject *parent, const
             QWidget *widget =  qobject_cast<QWidget*>(parent);
             return new VideoWidget(this, widget);
         }
-#endif //QT_NO_PHONON_VIDEO
+#ifndef PHONON_NO_GRAPHICSVIEW
+    case VideoGraphicsObjectClass:
+        return new VideoGraphicsObject(this, parent);
+#endif // PHONON_NO_GRAPHICSVIEW
+#endif // QT_NO_PHONON_VIDEO
 #ifndef QT_NO_PHONON_VOLUMEFADEREFFECT
     case VolumeFaderEffectClass:
         return new VolumeFaderEffect(this, parent);
-#endif //QT_NO_PHONON_VOLUMEFADEREFFECT
+#endif // QT_NO_PHONON_VOLUMEFADEREFFECT
 
     case VisualizationClass:  //Fall through
     default:
-        logMessage("createObject() : Backend object not available");
+        warning() << "Backend class" << c << "is not supported by Phonon GST :(";
     }
     return 0;
 }
@@ -206,16 +212,15 @@ bool Backend::checkDependencies(bool retry) const
                 gst_update_registry();
                 checkDependencies(true);
             }
-            QString message = tr("Warning: You do not seem to have the package gstreamer0.10-plugins-good installed.\n"
-                                 "          Some video features have been disabled.");
-            debug() << message;
+            warning() << tr("Warning: You do not seem to have the package gstreamer0.10-plugins-good installed.\n"
+                            "          Some video features have been disabled.");
         }
     } else {
         if (!retry) {
             gst_update_registry();
             checkDependencies(true);
         }
-        qWarning() << tr("Warning: You do not seem to have the base GStreamer plugins installed.\n"
+        warning() << tr("Warning: You do not seem to have the base GStreamer plugins installed.\n"
                          "          All audio and video support has been disabled");
     }
     return success;
@@ -304,21 +309,11 @@ QList<int> Backend::objectDescriptionIndexes(ObjectDescriptionType type) const
         return list;
 
     switch (type) {
-    case Phonon::AudioOutputDeviceType: {
-            QList<AudioDevice> deviceList = deviceManager()->audioOutputDevices();
-            for (int dev = 0 ; dev < deviceList.size() ; ++dev)
-                list.append(deviceList[dev].id);
-            break;
-        }
+    case Phonon::AudioOutputDeviceType:
+    case Phonon::AudioCaptureDeviceType:
+    case Phonon::VideoCaptureDeviceType:
+        list = deviceManager()->deviceIds(type);
         break;
-    case Phonon::VideoCaptureDeviceType: {
-            QList<VideoCaptureDevice> deviceList = deviceManager()->videoCaptureDevices();
-            for(int dev = 0 ; dev < deviceList.size() ; ++dev)
-                list.append(deviceList[dev].id);
-            break;
-        }
-        break;
-
     case Phonon::EffectType: {
             QList<EffectInfo*> effectList = effectManager()->audioEffects();
             for (int eff = 0 ; eff < effectList.size() ; ++eff)
@@ -326,9 +321,11 @@ QList<int> Backend::objectDescriptionIndexes(ObjectDescriptionType type) const
             break;
         }
         break;
-    case Phonon::SubtitleType: {
-            list << GlobalSubtitles::instance()->globalIndexes();
-        }
+    case Phonon::SubtitleType:
+        list << GlobalSubtitles::instance()->globalIndexes();
+        break;
+    case Phonon::AudioChannelType:
+        list << GlobalAudioChannels::instance()->globalIndexes();
         break;
     default:
         break;
@@ -341,34 +338,17 @@ QList<int> Backend::objectDescriptionIndexes(ObjectDescriptionType type) const
  */
 QHash<QByteArray, QVariant> Backend::objectDescriptionProperties(ObjectDescriptionType type, int index) const
 {
-
     QHash<QByteArray, QVariant> ret;
 
     if (!isValid())
         return ret;
 
     switch (type) {
-    case Phonon::AudioOutputDeviceType: {
-            AudioDevice* ad;
-            if ((ad = deviceManager()->audioDevice(index))) {
-                ret.insert("name", ad->gstId);
-                ret.insert("description", ad->description);
-                ret.insert("icon", ad->icon);
-            }
-        }
-        break;
-    case Phonon::VideoCaptureDeviceType: {
-            VideoCaptureDevice* dev;
-            if ((dev = deviceManager()->videoCaptureDevice(index))) {
-                ret.insert("name", dev->gstId);
-                ret.insert("description", dev->description);
-                ret.insert("icon", dev->icon);
-                DeviceAccessList devlist;
-                devlist << DeviceAccess("v4l2", dev->gstId);
-                ret.insert("deviceAccessList", QVariant::fromValue<Phonon::DeviceAccessList>(devlist));
-                ret.insert("hasvideo", true);
-            }
-        }
+    case Phonon::AudioOutputDeviceType:
+    case Phonon::AudioCaptureDeviceType:
+    case Phonon::VideoCaptureDeviceType:
+        // Index should be unique, even for different categories
+        ret = deviceManager()->deviceProperties(index);
         break;
 
     case Phonon::EffectType: {
@@ -391,6 +371,14 @@ QHash<QByteArray, QVariant> Backend::objectDescriptionProperties(ObjectDescripti
         }
         break;
 
+    case Phonon::AudioChannelType: {
+            const AudioChannelDescription description = GlobalAudioChannels::instance()->fromIndex(index);
+            ret.insert("name", description.name());
+            ret.insert("description", description.description());
+            ret.insert("type", description.property("type"));
+        }
+        break;
+
     default:
         break;
     }
@@ -422,12 +410,12 @@ bool Backend::connectNodes(QObject *source, QObject *sink)
         MediaNode *sinkNode = qobject_cast<MediaNode *>(sink);
         if (sourceNode && sinkNode) {
             if (sourceNode->connectNode(sink)) {
-                logMessage(QString("Backend connected %0 to %1").arg(source->metaObject()->className()).arg(sink->metaObject()->className()));
+                debug() << "Backend connected" << source->metaObject()->className() << "to" << sink->metaObject()->className();
                 return true;
             }
         }
     }
-    logMessage(QString("Linking %0 to %1 failed").arg(source->metaObject()->className()).arg(sink->metaObject()->className()), Warning);
+    warning() << "Linking" << source->metaObject()->className() << "to" << sink->metaObject()->className() << "failed";
     return false;
 }
 
@@ -470,59 +458,7 @@ EffectManager* Backend::effectManager() const
     return m_effectManager;
 }
 
-/**
- * Returns a debuglevel that is determined by the
- * PHONON_BACKEND_DEBUG environment variable.
- *
- *  Warning - important warnings
- *  Info    - general info
- *  Debug   - gives extra info
- */
-Backend::DebugLevel Backend::debugLevel() const
-{
-    return m_debugLevel;
-}
-
-/***
- * Prints a conditional debug message based on the current debug level
- * If obj is provided, classname and objectname will be printed as well
- *
- * see debugLevel()
- */
-void Backend::logMessage(const QString &message, int priority, QObject *obj) const
-{
-    // Backend is a singleton, so this is just fine.
-    static QString lastLogMessage = QString();
-    static int logMessageSkipCount = 0;
-    
-    if (debugLevel() > 0) {
-        QString output;
-        if (obj) {
-            // Strip away namespace from className
-            QString className(obj->metaObject()->className());
-            int nameLength = className.length() - className.lastIndexOf(':') - 1;
-            className = className.right(nameLength);
-            output.sprintf("%s %s (%s %p)", message.toLatin1().constData(),
-                                          obj->objectName().toLatin1().constData(),
-                                          className.toLatin1().constData(), obj);
-        }
-        else {
-            output = message;
-        }
-        if (priority <= (int)debugLevel() && lastLogMessage != output) {
-            if (logMessageSkipCount != 0) 
-                debug() << "  PGST: Last message repeated" << logMessageSkipCount << "time(s)";
-            debug() << QString("PGST(%1): %2").arg(priority).arg(output);
-            lastLogMessage = output;
-            logMessageSkipCount = 0;
-        } else
-            ++logMessageSkipCount;
-    }
-}
-
 }
 }
 
-QT_END_NAMESPACE
-
 #include "moc_backend.cpp"
