@@ -1,6 +1,6 @@
 /*  This file is part of the KDE project.
 
-    Copyright (C) 2011 Trever Fischer <tdfischer@fedoraproject.org>
+    Copyright (C) 2011 Torrie Fischer <tdfischer@kde.org>
 
     This library is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -24,8 +24,6 @@
 #include <phonon/MediaController>
 #include <QtCore/QMutex>
 
-QT_BEGIN_NAMESPACE
-
 typedef QMultiMap<QString, QString> TagMap;
 
 namespace Phonon
@@ -72,6 +70,7 @@ class Pipeline : public QObject
 
         static void cb_videoChanged(GstElement *playbin, gpointer data);
         static void cb_textTagsChanged(GstElement *playbin, gint stream, gpointer data);
+        static void cb_audioTagsChanged(GstElement *playbin, gint stream, gpointer data);
 
         GstElement *audioPipe();
         GstElement *videoPipe();
@@ -107,6 +106,7 @@ class Pipeline : public QObject
         void stateChanged(GstState oldState, GstState newState);
         void videoAvailabilityChanged(bool);
         void textTagChanged(int stream);
+        void audioTagChanged(int stream);
         void errorMessage(const QString &message, Phonon::ErrorType type);
         // Only emitted when metadata changes in the middle of a stream.
         void metaDataChanged(QMultiMap<QString, QString>);
@@ -120,7 +120,7 @@ class Pipeline : public QObject
         GstPipeline *m_pipeline;
         int m_bufferPercent;
 
-        // Keeps track of wether or not we jump to GST_STATE_PLAYING after plugin installtion is finished.
+        // Keeps track of whether or not we jump to GST_STATE_PLAYING after plugin installtion is finished.
         // Otherwise, it is possible to jump to another track, play a few seconds, pause, then finish installation
         // and spontaniously start playback without user action.
         bool m_resumeAfterInstall;
